# Blastoff — The Cosmos Hub Mainnet is Live
**Announcing the Mainnet Launch of the Cosmos Hub**

## About Cosmos
In 2014, Jae Kwon observed that the safety-favoring nature of classical Byzantine Fault Tolerant (BFT) consensus algorithms would allow us to use Proof of Stake (PoS) for Sybil resistance in a public blockchain context without the need for external sources of pseudorandomness. In 2015, he was joined by Ethan Buchman to build a future where there are as many blockchains as there are web apps today.

In order to prove that we could have a secure, BFT system running on Proof-of-Stake, we needed to test the theory out on a public, decentralized blockchain. That blockchain is the Cosmos Hub.

The core motivation fueling this launch is the desire to provide infrastructure for a scalable public blockchain network. The Cosmos vision is to enable communication and connectivity among thousands of cryptocurrency systems, allowing the network to scale without utilizing Proof-of-Work.

To make this vision a reality, [Tendermint Core](https://github.com/tendermint/tendermint) was developed as a BFT consensus engine that could be used as the first layer for any PoS blockchain.

The vision also required a framework to significantly lower the barrier to entry for developing decentralized applications. Similar to how web frameworks such as Ruby on Rails or Django brought about a Cambrian explosion of web applications, the [Cosmos SDK](https://github.com/cosmos/cosmos-sdk) was created to make blockchain development easy and catalyze the explosion of blockchain applications.

Nearly five years since the creation of Tendermint Core, the dream of creating an Internet of Blockchains powered by BFT Proof-of-Stake has become a reality. Here’s a look back at a brief history of the development progress which set the gears in motion toward our journey to the Cosmos.

Get an intro to Cosmos and its flagship open-source software here: [cosmos.network/intro](https://cosmos.network/intro)

Learn how to program custom application-specific blockchains with the Cosmos SDK by following [this tutorial](https://cosmos.network/docs/tutorial/).

### Cosmos Mainnet Launch
The Cosmos Hub is the first public BFT blockchain powered by proof-of-stake, and the first BFT system deployed at scale in almost 40 years of Byzantine Fault Tolerance (BFT) research history. The mainnet is set to be rolled out in three phases: network gains stability, transfers enabled, and IBC enabled.

- Read what Coindesk [has to say](https://www.coindesk.com/a-blockchain-to-connect-all-blockchains-cosmos-is-now-officially-live) about the Cosmos Hub launch.

Learn more about Cosmos here: [cosmos.network/about](https://cosmos.network/about)

## Cosmos Ecosystem
Below are some highlights from the Cosmos universe.

### Tendermint Inc Series A Round
Tendermint Inc, the lead developer of [Tendermint Core](https://github.com/tendermint/tendermint), [Cosmos SDK](https://github.com/cosmos/cosmos-sdk), and the [Cosmos Network](https://cosmos.network/), has raised its Series A in a round led by Paradigm’s USD $400 million blockchain fund. Other prominent investors like [Bain Capital](https://www.baincapitalventures.com/) and [1confirmation](https://www.1confirmation.com/) also participated. Part of the funds will be used by Tendermint Inc to support the continued development of the Cosmos Network and its ecosystem.

[Read more here](https://medium.com/tendermint/tendermint-inc-announces-series-a-round-9c062d1bd7de)

### Interchain Foundation Research & Development
[Interchain Foundation](https://interchain.io/) (ICF), the Swiss non-profit funding development of the Cosmos Network technology, is growing its research and development efforts by [collaborating with other research institutions](https://interchain.io/research) and [growing its internal R&D team](https://interchain.io/careers). It is also [accepting applications](https://interchain.typeform.com/to/P3tOm0) from projects seeking funding.

### Ecosystem & Use Cases
While the Cosmos Hub is the first blockchain launched in the Cosmos Network, the wider Cosmos project, like the observable universe, is quickly expanding. The number of contributors to the core software repositories has grown to over 150. The number of validators on testnets has been well over 100; the number of validators participating in the decentralized main net launch is over 50. There is a community-curated list where [80 projects](https://forum.cosmos.network/t/list-of-projects-in-cosmos-tendermint-ecosystem/243) and counting have indicated they are building, or intend to build, applications with the Cosmos SDK and/or Tendermint Core. Some of the highest profile projects can be found on the [ecosystem page](https://cosmos.network/ecosystem), and include applications in gaming, content distribution, music, ecological monitoring and regeneration, blockchain interoperability, and more — the wide diversity of use cases made possible by the modular and general purpose infrastructure that we’ve open-sourced.

Here’s a list of Cosmos SDK/Tendermint Core-based projects: [cosmos.network/ecosystem](https://cosmos.network/ecosystem)

### What our partners say about us
“The Cosmos team has created an essential piece of infrastructure that has the potential to ignite exponential growth in crypto-economic networks.” — Brian Kelly, BKCM

“We are excited to support Cosmos’s vision for an ecosystem of interoperable blockchains. The team, led by Jae Kwon, has shown years of dedication and a keen understanding for trade-offs in decentralized system design. We are particularly excited with the team’s progress in building the first production-ready Proof of Stake BFT system, and we look forward to seeing what new decentralized applications that enables.” — Matt Huang & Charlie Noyes, Paradigm
